from subs2cia.sources import AVSFile
from subs2cia.Common import Common, interactive_picker, chapter_timestamps
import subs2cia.subtools as subtools
from subs2cia.ffmpeg_tools import ffmpeg_trim_audio_clip_atrim_encode, ffmpeg_get_frame_fast, ffmpeg_trim_video_clip_directcopy

from typing import List, Union
from pathlib import Path
import logging
import tqdm
import pandas as pd
import unicodedata as ud
from collections import defaultdict

class CardExport(Common):
    def __init__(self, sources: List[AVSFile], outdir: Path, outstem: Union[str, None], condensed_video: bool, padding: int,
                 demux_overwrite_existing: bool, overwrite_existing_generated: bool,
                 keep_temporaries: bool, target_lang: str, out_audioext: str,
                 use_all_subs: bool, subtitle_regex_filter: str,
                 # subtitle_regex_substrfilter: str,
                 # subtitle_regex_substrfilter_nokeep: bool,
                 audio_stream_index: int, subtitle_stream_index: int,
                 ignore_range: Union[List[List[int]], None], ignore_chapters: Union[List[str], None],
                 bitrate: Union[int, None], mono_channel: bool, interactive: bool, normalize_audio: bool, out_audiocodec: str):
        super(CardExport, self).__init__(
            sources=sources,
            outdir=outdir,
            outstem=outstem,
            condensed_video=condensed_video,
            padding=padding,
            demux_overwrite_existing=demux_overwrite_existing,
            overwrite_existing_generated=overwrite_existing_generated,
            keep_temporaries=keep_temporaries,
            target_lang=target_lang,
            out_audioext=out_audioext,
            use_all_subs=use_all_subs,
            subtitle_regex_filter=subtitle_regex_filter,
            # subtitle_regex_substrfilter=subtitle_regex_substrfilter,
            # subtitle_regex_substrfilter_nokeep=subtitle_regex_substrfilter_nokeep,
            audio_stream_index=audio_stream_index,
            subtitle_stream_index=subtitle_stream_index,
            ignore_range=ignore_range,
            ignore_chapters=ignore_chapters,
            bitrate=bitrate,
            mono_channel=mono_channel,
            interactive=interactive,
            out_audiocodec=out_audiocodec
        )

        self.insufficient = False

        self.normalize_audio = normalize_audio

        self.subdata = None

    def choose_subtitle(self, interactive):
        if len(self.partitioned_streams['subtitle']) == 0:
            logging.warning(f"Couldn't find audio streams in input files")
            return
        # if interactive and len(self.partitioned_streams['subtitle']) > 1:
        #     while self.picked_streams['subtitle'] is None:
        #         self.picked_streams['subtitle'] = interactive_picker(self.sources, self.partitioned_streams, 'subtitle')
        #     return

        k = 'subtitle'
        while self.picked_streams[k] is None:
            if interactive and len(self.partitioned_streams['subtitle']) > 1:
                self.picked_streams['subtitle'] = interactive_picker(self.sources, self.partitioned_streams,
                                                                     'subtitle')
            else:
                try:
                    self.picked_streams[k] = next(self.pickers[k])
                except StopIteration as s:
                    logging.critical(f"Input files {self.sources} don't contain usable subtitles")
                    self.insufficient = True
                    return
            subfile = self.picked_streams[k].demux(overwrite_existing=self.demux_overwrite_existing)
            ignore_range = (self.ignore_range or []) + chapter_timestamps(self.picked_streams[k].file, self.ignore_chapters or [])
            if subfile is None:
                logging.warning(f"Error while demuxing {self.picked_streams[k]}")
                self.picked_streams[k] = None

            audiolength = subtools.get_audiofile_duration(self.picked_streams['audio'].demux_file.filepath)
            subdata = subtools.SubtitleManipulator(subfile.filepath,
                                                   threshold=0, padding=self.padding,
                                                   ignore_range=ignore_range, audio_length=audiolength)
            subdata.load(include_all=self.use_all_subs, regex=self.subtitle_regex_filter,
                         substrreplace_regex='', substrreplace_nokeepchanges=False)
            if subdata.ssadata is None:
                logging.warning(f"Problem loading subtitle data from {self.picked_streams[k]}")
                self.picked_streams[k] = None
                continue

            self.subdata = subdata

    def export(self):
        # expose these as options at some point
        # will need to rename these if they are going to become cli switches so they don't conflict
        forbidden_chars = ['[' , ']' , '<' , '>' , ':' , '"' , '/' , '?' , '*' , '^' , '\\' , '|']  # see fn disallowed_char in https://github.com/ankitects/anki/blob/main/rslib/src/media/files.rs
        forbidden_chars = {ord(c): '' for c in forbidden_chars}
        export_audio = True if self.picked_streams['audio'] is not None else False
        export_screenshot = True if self.picked_streams['video'] is not None else False
        export_video = False
        lbda = 0.0  # where to get screenshot. 0=start, 1=end, 0.5=middle
        w = -1
        h = -1
        columns = ['text', 'timestamps', 'audioclip', 'screenclip', 'videoclip', 'sources']
        exported = pd.DataFrame(columns=columns)
        for group in tqdm.tqdm(self.subdata.groups):
            # since subdata.merge_groups hasn't been called, each group only contains one SSAevent

            if group.contains_only_ephemeral:
                continue
            row = {'text': group.events[0].plaintext,
                   'timestamps': f"{group.group_range[0]}-{group.group_range[1]}",
                   'audioclip': None,
                   'screenclip': None,
                   'videoclip': None,
                   'sources': ",".join([s.filepath.name for s in self.sources])}
            if export_audio:
                outpath = self.outdir / (ud.normalize('NFC', self.outstem).translate(forbidden_chars) + f"_{group.group_range[0]}-{group.group_range[1]}.mp3")
                row['audioclip'] = f"[sound:{outpath.name}]"
                ffmpeg_trim_audio_clip_atrim_encode(input_file=self.picked_streams['audio'].demux_file.filepath,
                                                    stream_index=0,
                                                    timestamp_start=group.group_range[0],
                                                    timestamp_end=group.group_range[1], quality=self.quality,
                                                    to_mono=self.to_mono, normalize_audio=self.normalize_audio,
                                                    outpath=outpath)
            if export_screenshot:
                outpath = self.outdir / (ud.normalize('NFC', self.outstem).translate(forbidden_chars) + f"_{group.group_range[0]}-{group.group_range[1]}.jpg")
                row['screenclip'] = f"<img src='{outpath.name}'>"
                timestamp = (1-lbda) * group.group_range[0] + lbda * group.group_range[1]
                ffmpeg_get_frame_fast(self.picked_streams['video'].file.filepath,
                                      timestamp=timestamp, outpath=outpath, w=w, h=h)
            if export_video:
                outpath = self.outdir / (ud.normalize('NFC', self.outstem).translate(forbidden_chars) + f"_{group.group_range[0]}-{group.group_range[1]}.mp4")
                row['videoclip'] = f"[sound:{outpath.name}]"
                ffmpeg_trim_video_clip_directcopy(self.picked_streams['video'].file.filepath,
                                                  timestamp_start=group.group_range[0],
                                                    timestamp_end=group.group_range[1], quality=None, outpath=outpath)
            exported = exported.append([row], ignore_index=True)
            # if exported.shape[0] > 10:  # DEBUG ONLY
            #     break
        # print(exported)
        outpath = self.outdir / (self.outstem + ".tsv")
        exported.to_csv(outpath, sep='\t', index=False, header=False)



#!/usr/bin/env python3
import os
import sys
import shutil
import glob

# this line is for when main.py is run directly
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from subs2cia.argparser import get_args_subs2cia
from subs2cia.sources import AVSFile, group_files
from subs2cia.condense import Condense
from subs2cia.CardExport import CardExport

from pathlib import Path
import logging
from colorlog import ColoredFormatter
from pprint import pprint
from typing import Union, List
import tqdm

presets = [
    {  # preset 0
        'preset_description': "Padded and merged Japanese condensed audio",
        'threshold': 1500,
        'padding': 200,
        # 'partition_size': 1800,  # 30 minutes, for long movies
        'target_lang': 'ja',
    },
    {  # preset 1
        'preset_description': "Unpadded Japanese condensed audio",
        'threshold': 0,  # note: default is 0
        'padding': 0,  # note: default is 0
        # 'partition': 1800,  # 30 minutes, for long movies
        'target_lang': 'ja',
    },
]


def list_presets():
    for idx, preset in enumerate(presets):
        print(f"Preset {idx}")
        pprint(preset)


# https://stackoverflow.com/a/38739634
class TqdmLoggingHandler(logging.Handler):
    def __init__(self, level=logging.NOTSET):
        super().__init__(level)

    def emit(self, record):
        try:
            msg = self.format(record)
            tqdm.tqdm.write(msg)
            self.flush()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)


def condense_start(args, groups: List[List[AVSFile]]):
    condense_args = {key: args[key] for key in
                     ['outdir', 'outstem', 'condensed_video', 'padding', 'threshold', 'partition', 'split',
                      'demux_overwrite_existing', 'overwrite_existing_generated', 'keep_temporaries',
                      'target_lang', 'out_audioext', 'minimum_compression_ratio', 'use_all_subs',
                      'subtitle_regex_filter',
                      'subtitle_regex_substrfilter', 'subtitle_regex_substrfilter_nokeep',
                      'audio_stream_index', 'subtitle_stream_index', 'ignore_range', 'ignore_chapters',
                      'bitrate', 'mono_channel', 'interactive', 'no_condensed_subtitles', 'out_audiocodec']}

    condensed_files = [Condense(g, **condense_args) for g in groups]
    if logging.root.isEnabledFor(logging.INFO):
        logging.info("Input/output file mapping:")
        for cgroup in condensed_files:
            logging.info(f"{cgroup.outstem}")
            for cfile in cgroup.sources:
                logging.info(f"    {cfile.filepath}")

    # logging.root.addHandler(TqdmLoggingHandler())

    i = condensed_files
    # if logging.root.level == logging.INFO:
    #     # logging level of WARNING means quiet output
    #     # debug is too noisy for a progress bar to be _that_ useful
    #     i = tqdm.tqdm(condensed_files, position=0)

    for idx, c in enumerate(i):
        # if logging.root.level == logging.INFO:
        #     i.set_postfix_str(f"{c.outstem}")
        #     i.update(0)
        logging.info(f"({idx + 1}/{len(i)}): {c.outstem}")
        c.get_and_partition_streams()
        c.initialize_pickers()
        if args['dry_run']:
            continue
        if args['list_streams']:
            c.list_streams()
            continue
        c.choose_streams()
        c.export()
        c.cleanup()


def srs_export_start(args, groups: List[List[AVSFile]]):
    srs_args = {key: args[key] for key in
                ['outdir', 'outstem', 'condensed_video', 'padding', 'demux_overwrite_existing',
                 'overwrite_existing_generated', 'keep_temporaries', 'target_lang', 'out_audioext', 'use_all_subs',
                 'subtitle_regex_filter', 'audio_stream_index', 'subtitle_stream_index', 'ignore_range',
                 'ignore_chapters',
                 'bitrate', 'mono_channel', 'interactive', 'normalize_audio', "out_audiocodec"]
                }

    cardexport_group = [CardExport(g, **srs_args) for g in groups]

    for c in cardexport_group:
        c.get_and_partition_streams()
        c.initialize_pickers()
        if args['dry_run']:
            continue
        if args['list_streams']:
            c.list_streams()
            continue
        c.choose_streams()
        c.export()
        c.cleanup()


def start():
    if not shutil.which('ffmpeg'):
        logging.warning(f"Couldn't find ffmpeg in PATH, things may break.")

    args = get_args_subs2cia()
    args = vars(args)

    logconfig = False
    ch = logging.StreamHandler()
    level = logging.NOTSET
    if args['debug']:
        level = logging.DEBUG
        logconfig = True
    if not logconfig and args['quiet']:
        level = logging.WARNING
        logconfig = True
    if not logconfig:
        level = logging.INFO
        logconfig = True

    if level == logging.DEBUG:
        format = "subs2cia:%(log_color)s%(levelname)s%(reset)s:%(message_log_color)s%(message)s [%(module)s.py:%(funcName)s():%(lineno)d]"
    else:
        format = "subs2cia:%(log_color)s%(levelname)s%(reset)s:%(message_log_color)s%(message)s"

    ch.setLevel(level)
    ch.setFormatter(ColoredFormatter(format, log_colors={
        'DEBUG': 'cyan',
        'INFO': 'green',
        'WARNING': 'yellow',
        'ERROR': 'red',
        'CRITICAL': 'bold_red,bg_white',
    }, secondary_log_colors={
        'message': {
            'ERROR': 'red',
            'CRITICAL': 'bold_red'
        }
    }))
    logging.root.setLevel(level)
    logging.root.addHandler(ch)

    from subs2cia import __version__
    logging.info(f"subs2cia version {__version__}")
    logging.debug(f"Start arguments: {args}")

    if args['list_presets']:  # todo: user-defined presets
        list_presets()
        return

    if args['preset'] is not None:
        if abs(args['preset']) >= len(presets):
            logging.critical(f"Preset {args['preset']} does not exist")
            exit(0)
        logging.info(f"Using preset {args['preset']}")
        for key, val in presets[args['preset']].items():
            if key in args.keys() and ((args[key] == False) or (args[key] is None)):  # override presets
                args[key] = val

    if args['infiles'] is None:
        logging.warning("No input files given, nothing to do.")
        exit(0)

    infiles = _resolve(args['infiles'])

    # convert to Path objects and see if any input files are actually directories
    # for the moment, don't support processing entire directories.
    # point users to globbing and asterisks if they want to process an entire directory
    infiles = [Path(file) for file in infiles]
    for file in infiles:
        if file.is_dir():
            logging.error(f'Input file "{file}" is actually a directory, aborting.\n'
                          f'If you want to process all files in this directory, use "{file}/*" instead.')
            exit(2)

    if args['absolute_paths']:
        sources = [AVSFile(Path(file).absolute()) for file in infiles]
    else:
        sources = [AVSFile(Path(file)) for file in infiles]

    for s in sources:
        s.probe()
        s.get_type()

    if args['batch']:
        args['outstem'] = None
        logging.info(f"Running in batch mode, attempting to group similarly named files together.")
        groups = list(group_files(sources))
    else:
        if len(sources) > 2:
            logging.warning(f"Redundant input files detected. Got {len(sources)} "
                            f"input files to process and not running "
                            f"in batch mode. Only one output "
                            f"will be generated. ")
        groups = [list(sources)]
    logging.debug(f"Have {len(groups)} group(s) to process.")

    commands = {
        'condense': condense_start,
        'srs': srs_export_start
    }
    commands[args['command']](args, groups)


def _resolve(files):
    resolved = []

    for f in files:
        if '*' in f or '?' in f:
            globbed = glob.glob(f)
            if len(globbed) > 0:
                resolved += globbed
            else:
                resolved.append(f)
        else:
            resolved.append(f)

    return resolved


if __name__ == '__main__':
    start()



from subs2cia.argparser import get_args_subzipper
from subs2cia.sources import is_language

import logging
from pathlib import Path
import pycountry
from pprint import pprint


def start():
    args = get_args_subzipper()
    args = vars(args)

    if args['verbose']:
        # if args['debug']:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)
    # elif args['debug']:
    #     logging.basicConfig(level=logging.DEBUG)

    logging.debug(f"Start arguments: {args}")

    subfiles = args['subfiles']
    reffiles = args['reffiles']

    if len(subfiles) != len(reffiles):
        logging.warning(f"Mismatched number of subtitle and reference files! Got {len(subfiles)} subtitle files and "
                        f"{len(reffiles)} reference files.")
        logging.warning(f"Will only process the first "
                        f"{len(subfiles) if len(subfiles) < len(reffiles) else len(reffiles)} "
                        f"reference-subtitle pairs.")
        # exit(1)

    # subfiles = [Path(s).absolute() for s in subfiles]
    # reffiles = [Path(r).absolute() for r in reffiles]
    subfiles = [Path(s) for s in subfiles]
    reffiles = [Path(r) for r in reffiles]

    lang = None
    if args['lang'] is not None:
        if is_language(args['lang']):
            lang = pycountry.languages.lookup(args['lang'])
            lang = lang.alpha_3
            logging.info(f'Appending language code {lang}')
        else:
            logging.error(f"Language lookup failure: {args['lang']} is not a ISO recognized language")

    if args['no_sort']:
        logging.info("Not sorting inputs alphabetically, using as-is.")
    else:
        subfiles.sort(key=lambda x: str(x))
        reffiles.sort(key=lambda x: str(x))

    for s, r in zip(subfiles, reffiles):
        newpath = r.parent / (r.stem + ('' if lang is None else f'.{lang}') + s.suffix)
        logging.info(f"Will rename {s} to {newpath}")

        if not s.exists():
            logging.critical(f"Subtitle file doesn't exist: {s}")
            exit(1)
        if not r.exists():
            logging.warning(f"Reference file doesn't exist: {r}")
        if newpath == r:
            logging.critical(f"Renaming subtitle to {newpath} will overwrite the reference file!")
            exit(1)
        if newpath.exists():
            logging.critical(f"Renaming subtitle to {newpath} will overwrite an existing file!")
            exit(1)

    # todo: user-interactive question here
    if args['dry_run']:
        logging.info("Dry run mode, not writing changes.")
        return

    for s, r in zip(subfiles, reffiles):
        newpath = r.parent / (r.stem + ('' if lang is None else f'.{lang}') + s.suffix)
        logging.info(f"Renaming {s} to {newpath}...")
        s.rename(newpath)
        logging.info(f"...done")


if __name__ == '__main__':
    start()



from __future__ import unicode_literals, print_function
import logging
from pathlib import Path
import subprocess
from typing import List, Union
import gevent, gevent.monkey
from tqdm import tqdm, TqdmWarning

import warnings
warnings.filterwarnings("ignore", category=TqdmWarning)

import contextlib
import ffmpeg
import gevent
import gevent.monkey; gevent.monkey.patch_all(thread=False)
import os
import shutil
import socket
import sys
import tempfile
import textwrap

import json


@contextlib.contextmanager
def _tmpdir_scope():
    tmpdir = tempfile.mkdtemp()
    try:
        yield tmpdir
    finally:
        shutil.rmtree(tmpdir)


def _do_watch_progress(filename, sock, handler):
    """Function to run in a separate gevent greenlet to read progress
    events from a unix-domain socket."""
    connection, client_address = sock.accept()
    data = b''
    try:
        while True:
            more_data = connection.recv(16)
            if not more_data:
                break
            data += more_data
            lines = data.split(b'\n')
            for line in lines[:-1]:
                line = line.decode()
                parts = line.split('=')
                key = parts[0] if len(parts) > 0 else None
                value = parts[1] if len(parts) > 1 else None
                handler(key, value)
            data = lines[-1]
    finally:
        connection.close()


@contextlib.contextmanager
def _watch_progress(handler):
    """Context manager for creating a unix-domain socket and listen for
    ffmpeg progress events.
    The socket filename is yielded from the context manager and the
    socket is closed when the context manager is exited.
    Args:
        handler: a function to be called when progress events are
            received; receives a ``key`` argument and ``value``
            argument. (The example ``show_progress`` below uses tqdm)
    Yields:
        socket_filename: the name of the socket file.
    """
    with _tmpdir_scope() as tmpdir:
        socket_filename = os.path.join(tmpdir, 'sock')
        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        with contextlib.closing(sock):
            sock.bind(socket_filename)
            sock.listen(1)
            child = gevent.spawn(_do_watch_progress, socket_filename, sock, handler)
            try:
                yield socket_filename
            except:
                gevent.kill(child)
                raise



@contextlib.contextmanager
def show_progress(total_duration, desc):
    """Create a unix-domain socket to watch progress and render tqdm
    progress bar."""
    with tqdm(total=round(total_duration, 2), position=0, unit='sec', desc=desc) as bar:
        def handler(key, value):
            if key == 'out_time_ms':
                time = round(float(value) / 1000000., 2)
                bar.update(time - bar.n)
            elif key == 'progress' and value == 'end':
                bar.update(bar.total - bar.n)
        with _watch_progress(handler) as socket_filename:
            yield socket_filename


# given a stream in the input file, demux the stream and save it into the outfile with some type
def ffmpeg_demux(infile: Path, stream_idx: int, outfile: Path):
    # output format is specified via extention on outfile
    logging.debug(f"demuxing stream {stream_idx} from file {infile} to {outfile}")
    video = ffmpeg.input(str(infile))
    stream = video[str(stream_idx)]  # don't need 0
    stream = ffmpeg.output(stream, str(outfile))
    stream = ffmpeg.overwrite_output(stream)
    logging.debug(f"ffmpeg arguments: {ffmpeg.get_args(stream)}")

    quiet = not logging.root.isEnabledFor(logging.DEBUG)

    try:
        ffmpeg.run(stream, quiet=quiet)  # verbose only
    except ffmpeg.Error as e:
        if e.stderr is None:
            logging.warning(
                f"Couldn't demux stream {stream_idx} from {infile}, skipping.")
            return None
        logging.warning(
            f"Couldn't demux stream {stream_idx} from {infile}, skipping. ffmpeg output: \n" + e.stderr.decode("utf-8"))
        return None
    return outfile


# from ffmpeg-python _run.py
class Error(Exception):
    def __init__(self, cmd, stdout, stderr: bytes):
        super(Error, self).__init__(
            '{} error (see stderr output for detail)'.format(cmd)
        )
        self.stdout = stdout
        self.stderr = stderr


def ffmpeg_condense_audio(audiofile, sub_times, quality: Union[int, None], to_mono: bool, outfile=None, codec=''):
    if outfile is None:
        outfile = "condensed.flac"
    # logging.info(f"saving condensed audio to {outfile}")

    # get samples in audio file
    # prefer codec_time_base because some files will have different values for each, and codec_time_base seems to be the
    #  most accurate
    audio_info = ffmpeg.probe(audiofile, cmd='ffprobe')
    if 'codec_time_base' in audio_info['streams'][0]:
        # audio samples per second, inverse of sampling frequency
        sps = int(
            audio_info['streams'][0]['codec_time_base'].split('/')[1])
    elif 'time_base' in audio_info['streams'][0]:
        # audio samples per second, inverse of sampling frequency
        sps = int(
            audio_info['streams'][0]['time_base'].split('/')[
                1])
    else:
        info = json.dumps(audio_info['streams'][0])
        logging.error("ffprobe couldn't determine audio time_base, can't condense")
        raise Error("", '', info.encode('utf-8'))
    # samples = audio_info['streams'][0]['duration_ts']  # total samples in audio track
    # duration_ts uses time_base, not codec_time_base

    stream = ffmpeg.input(audiofile)

    clips = list()
    for time in sub_times:  # times are in milliseconds
        start = int(time[0] * sps / 1000)  # convert to sample index
        end = int(time[1] * sps / 1000)
        # use start_pts for sample/millisecond level precision
        clips.append(stream.audio.filter('atrim', start_pts=start, end_pts=end).filter('asetpts', 'PTS-STARTPTS'))
    combined = ffmpeg.concat(*clips, a=1, v=0)

    kwargs = {}

    if codec:
        kwargs['acodec'] = codec

    if Path(outfile).suffix.lower() == ".mp3" or codec == 'mp3':
        if quality is None:
            kwargs['audio_bitrate'] = "320k"
        else:
            kwargs['audio_bitrate'] = f"{quality}k"
    if to_mono:
        kwargs['ac'] = 1

    combined = ffmpeg.output(combined, outfile, **kwargs)
    combined = ffmpeg.overwrite_output(combined)

    if logging.root.isEnabledFor(logging.DEBUG):
        logging.debug(f"ffmpeg_condense_audio: ffmpeg arguments: {' '.join(ffmpeg.get_args(combined))}")

    duration = sum([end - start for start, end in sub_times]) / 1000
    ffmpeg_exec(duration, outfile, combined)


def export_condensed_audio(divided_times, audiofile: Path, quality: Union[int, None], to_mono: bool, outfile=None,
                           use_absolute_numbering=False, codec=''):
    # outfile is full path with extension
    audiofile = str(audiofile)
    if outfile is not None:
        outfile = str(outfile)

    if outfile is None:  # no output path given, use audiofile path
        outfile = audiofile
    elif outfile[0] == '.' and outfile[1:].isalnum():  # outfile is just an extension, use audiofile for path
        # extension = outfile
        outfile = os.path.splitext(audiofile)[0] + outfile
    else:  # outfile is already full path with extension
        pass
    idx = 0
    for p, partition in enumerate(divided_times):
        if len(partition) == 0:
            continue
        for s, split in enumerate(partition):
            if len(split) == 0:
                continue
            idx += 1
            if use_absolute_numbering:  # todo: remove outfile naming from this function
                outfilesplit = os.path.splitext(outfile)[0] + \
                               f".pt{idx}" + \
                               ".condensed" + \
                               os.path.splitext(outfile)[1]
            else:
                outfilesplit = os.path.splitext(outfile)[0] + \
                               (f".p{p + 1}" if len(divided_times) != 1 else "") + \
                               (f".s{s + 1}" if len(partition) != 1 else "") + \
                               ".condensed" + \
                               os.path.splitext(outfile)[1]
            try:
                ffmpeg_condense_audio(audiofile=audiofile, sub_times=split, outfile=outfilesplit, quality=quality,
                                      to_mono=to_mono, codec=codec)
                logging.info(f"Wrote condensed audio to {outfilesplit}")
            except Error as e:
                logging.error(
                    f"ffmpeg couldn't export audio. ffmpeg output: \n" + e.stderr.decode("utf-8"))


def export_condensed_video(divided_times, audiofile: Path, subfile: Path, videofile: Path, outfile=None,
                           use_absolute_numbering=False):
    # outfile is full path with extension
    audiofile = str(audiofile)
    if outfile is not None:
        outfile = str(outfile)

    if outfile is None:  # no output path given, use audiofile path
        outfile = audiofile
    elif outfile[0] == '.' and outfile[1:].isalnum():  # outfile is just an extension, use audiofile for path
        # extension = outfile
        outfile = os.path.splitext(audiofile)[0] + outfile
    else:  # outfile is already full path with extension
        pass
    idx = 0
    for p, partition in enumerate(divided_times):
        if len(partition) == 0:
            continue
        for s, split in enumerate(partition):
            if len(split) == 0:
                continue
            idx += 1
            if use_absolute_numbering:
                outfilesplit = os.path.splitext(outfile)[0] + \
                               f".pt{idx}" + \
                               ".condensed" + \
                               os.path.splitext(outfile)[1]
            else:
                outfilesplit = os.path.splitext(outfile)[0] + \
                               (f".p{p + 1}" if len(divided_times) != 1 else "") + \
                               (f".s{s + 1}" if len(partition) != 1 else "") + \
                               ".condensed" + \
                               os.path.splitext(outfile)[1]
            # todo: need to split subfiles with partition, split options
            try:
                ffmpeg_condense_video(audiofile=audiofile, videofile=str(videofile), subfile=str(subfile),
                                      sub_times=split, outfile=outfilesplit)
            except Error as e:
                logging.error(
                    f"ffmpeg couldn't export video. ffmpeg output: \n" + e.stderr.decode("utf-8"))


def trim(input_path, output_path, start=30, end=60):
    input_stream = ffmpeg.input(input_path)

    vid = (
        input_stream.video
            .trim(start=start, end=end)
            .setpts('PTS-STARTPTS')
    )
    aud = (
        input_stream.audio
            .filter_('atrim', start=start, end=end)
            .filter_('asetpts', 'PTS-STARTPTS')
    )

    joined = ffmpeg.concat(vid, aud, v=1, a=1).node
    output = ffmpeg.output(joined[0], joined[1], output_path)
    output.run()


def ffmpeg_exec(duration: float, outfile: str, combined):
    def run(combined):
        args = ffmpeg.get_args(combined)

        if len("ffmpeg " + " ".join(args)) > 30000:
            # logging.info("Arguments passed to ffmpeg exceeds 32767 characters while running on a Windows system. "
            #              "Will try using a temporary file to pass filter_complex arguments to ffmpeg.")
            logging.debug("ffmpeg command length exceeds 30000, will try writing to a temporary file")
            idx = args.index("-filter_complex") + 1
            complex_filter = str(args[idx])
            # write complex_filter to a temporary file
            fp = tempfile.NamedTemporaryFile(
                delete=False)  # don't delete b/c can't open file again when it's already open in windows
            fp.write(complex_filter.encode(encoding="utf-8"))
            fp.close()
            logging.debug(f"using temporary file {fp.name}")
            args[idx] = fp.name
            args[idx - 1] = "-filter_complex_script"
        args = ["ffmpeg"] + args  # + ['-progress', 'unix://{}'.format(socket_filename)]

        # ffmpeg.run(combined, quiet=logging.getLogger().getEffectiveLevel() >= logging.WARNING)

        pipe_stdin = False
        pipe_stdout = True
        pipe_stderr = True
        quiet = not logging.root.isEnabledFor(logging.DEBUG)

        stdin_stream = subprocess.PIPE if pipe_stdin else None
        stdout_stream = subprocess.PIPE if pipe_stdout or quiet else None
        stderr_stream = subprocess.PIPE if pipe_stderr or quiet else None
        process = subprocess.Popen(
            args, stdin=stdin_stream, stdout=stdout_stream, stderr=stderr_stream
        )
        out, err = process.communicate()
        retcode = process.poll()
        if retcode:
            raise Error('ffmpeg', out, err)

    # if os.name == 'posix':
    if hasattr(socket, 'AF_UNIX') and logging.root.level <= logging.INFO:
        with show_progress(total_duration=duration, desc=outfile) as socket_filename:
            combined = combined.global_args('-progress', 'unix://{}'.format(socket_filename))
            run(combined)
    else:
        run(combined)


def ffmpeg_condense_video(audiofile: str, videofile: str, subfile: str, sub_times, outfile):
    # logging.info(f"saving condensed video to {outfile}")

    # get samples in audio file
    audio_info = ffmpeg.probe(audiofile, cmd='ffprobe')
    sps = int(
        audio_info['streams'][0]['time_base'].split('/')[1])  # audio samples per second, inverse of sampling frequency
    # samples = audio_info['streams'][0]['duration_ts']  # total samples in audio track

    audiostream = ffmpeg.input(audiofile)
    videostream = ffmpeg.input(videofile)
    substream = ffmpeg.input(subfile)
    vid = videostream.video.filter_multi_output('split')
    # sub = videostream['s'].filter_multi_output('split')
    aud = audiostream.audio.filter_multi_output('asplit')

    clips = []
    for idx, time in enumerate(sub_times):  # times are in milliseconds
        # start = int(time[0] * sps / 1000)  # convert to sample index
        # end = int(time[1] * sps / 1000)
        start = time[0] / 1000
        end = time[1] / 1000
        # use start_pts for sample/millisecond level precision

        a = aud[idx].filter('atrim', start=start, end=end).filter('asetpts', expr='PTS-STARTPTS')
        v = vid[idx].trim(start=start, end=end).setpts('PTS-STARTPTS')
        # s = sub[idx].trim(start=start, end=end).setpts('PTS-STARTPTS')
        clips.extend((v, a))

    out = ffmpeg.concat(
        *clips,
        v=1,
        a=1
    ).output(substream, outfile)

    # output = ffmpeg.output(joined[0], joined[1], outfile)
    out = ffmpeg.overwrite_output(out)
    if logging.root.isEnabledFor(logging.DEBUG):
        logging.debug(f"ffmpeg_condense_video: ffmpeg arguments: {' '.join(ffmpeg.get_args(out))}")

    duration = sum([end - start for start, end in sub_times]) / 1000
    ffmpeg_exec(duration, outfile, out)


def ffmpeg_get_frames(videofile: Path, timestamps: List[int], outdir: Path, outstem: str, outext: str, w: int, h: int):
    r"""

    :param videofile:
    :param timestamps:
    :param outdir:
    :param outstem:
    :param outext: image extension. Should include dot.
    :return:
    """
    # todo: make async if needed
    for idx, timestamp in enumerate(timestamps):
        outname = outstem + f"_{idx}_{timestamp}" + outext
        ffmpeg_get_frame_fast(videofile, timestamp, outdir / outname, w, h)


# too slow, not used
def ffmpeg_get_frame(videofile: Path, timestamp: int, outpath: Path):
    # logging.debug(f"Saving frame from {videofile} at {timestamp}ms to {outpath}")
    videostream = ffmpeg.input(str(videofile))

    # from https://superuser.com/a/1330042
    # this method will probably need the windows long-argument fix as well
    # it's also relatively slow
    videostream = videostream.video.filter('select', f"lt(prev_pts*TB,{timestamp/1000})*gte(pts*TB,{timestamp/1000})")

    # from https://stackoverflow.com/a/28321986
    # TODO: compare speed of this and the other method

    videostream = ffmpeg.output(videostream, str(outpath), vsync='drop')
    args = videostream.get_args()
    logging.debug(f"ffmpeg_get_frame: args: {args}")
    ffmpeg.run(videostream)


def ffmpeg_get_frame_fast(videofile: Path, timestamp: int, outpath: Path, w: int, h: int,
                          format: Union[str, None] = None, capture_stdout: bool = False, silent: bool = True):
    r"""
    Gets a screenshot from a video file
    :param videofile: Path to input file containing a video stream
    :param timestamp: Timestamp to get in milliseconds
    :param outpath: Output path to save to. If set to 'pipe:', will instead pipe to stdout.
    :param w: Width in pixels. -1 preserves aspect ratio.
    :param h: Height in pixels. -1 preserves aspect ratio.
    :param format: Output format (e.g. mjpeg for JPG, image2 for PNG, etc), required if outpath is missing/misleading an extension.
    :param capture_stdout: If set to true, captures ffmpeg's stdout and returns as raw bytestream.
            Otherwise prints to stdout.
    :param silent: Suppresses stderr from printing if set. Useful for exporting many frames at a time.
    :return:
    """

    # from https://stackoverflow.com/a/28321986
    videostream = ffmpeg.input(str(videofile), ss=timestamp/1000)
    if w == -1 and h == -1:
        pass
    else:
        videostream = videostream.video.filter('scale', w, h)

    kwargs = {'vframes': 1}
    if format is not None:
        kwargs['format'] = format

    videostream = ffmpeg.output(videostream, str(outpath), **kwargs)
    videostream = ffmpeg.overwrite_output(videostream)
    args = videostream.get_args()
    logging.debug(f"ffmpeg_get_frame_fast: args: {args}")
    stdout, stderr = ffmpeg.run(videostream, capture_stdout=capture_stdout, capture_stderr=silent)

    return stdout


def ffmpeg_trim_audio_clip_directcopy(videofile: Path, stream_index: int, timestamp_start: int, timestamp_end: int, outpath: Path):
    videostream = ffmpeg.input(str(videofile))
    # outpath extension must be a container format (mp4, mkv) or the same type as the audio (.mp3, .eac3, .flac, etc)
    # todo: may need to use AVSfile to specify audio stream/direct demux
    videostream = ffmpeg.output(videostream[str(stream_index)], str(outpath), ss=timestamp_start/1000, to=timestamp_end/1000, c="copy")

    videostream = ffmpeg.overwrite_output(videostream)
    args = videostream.get_args()
    logging.debug(f"ffmpeg_trim_audio_clip: args: {args}")
    ffmpeg.run(videostream)


def ffmpeg_trim_audio_clip_atrim_encode(input_file: Path, stream_index: int, timestamp_start: int, timestamp_end: int,
                                        quality: Union[int, None], to_mono: bool, normalize_audio: bool,
                                        outpath: Path, format: str = None, capture_stdout: bool = False,
                                        silent: bool = True):
    r"""
    Take media file and export a trimmed audio file.
    :param stream_index: FFmpeg stream index. If input is not a container format, 0 should be used.
    :param capture_stdout: If true, returns stdout. Used in conjunction with outpath="pipe:" and format option.
    :param input_file: Path to video/audio file to clip from.
    :param timestamp_start: Start time in milliseconds.
    :param timestamp_end: End time in milliseconds.
    :param quality: If output extension is .mp3, this is the bitrate in kbps. Ignored otherwise.
    :param to_mono: If set, mixes all input channels to mono to save space
    :param normalize_audio: If set, attempts to normalize loudness of output audio. YMMV.
    :param outpath: Path to save to.
    :param format: Output format (e.g. mp3, flac, etc), required if extension of outpath is missing/not the intended
                    format. Required if outpath is "pipe:" since there is no output extension to infer from.
    :param silent: If set, suppresses stderr.
    :return: FFmpeg stdout data if capture_stdout is set
    """
    input_stream = ffmpeg.input(str(input_file))
    input_stream = input_stream[str(stream_index)]

    input_stream = input_stream.filter("atrim",
                                     start=timestamp_start/1000,
                                     end=timestamp_end/1000).filter("asetpts", "PTS-STARTPTS")

    if normalize_audio:
        input_stream = input_stream.filter("loudnorm", print_format="summary")

    kwargs = {}

    if outpath.suffix.lower() == ".mp3":
        if quality is not None:
            kwargs['audio_bitrate'] = f'{quality}k'
        else:
            kwargs['audio_bitrate'] = '320k'

    if to_mono:
        kwargs['ac'] = 1  # audio channels

    if format is not None:
        kwargs['format'] = format
    input_stream = ffmpeg.output(input_stream, str(outpath), **kwargs)


    input_stream = ffmpeg.overwrite_output(input_stream)
    args = input_stream.get_args()
    # logging.debug(f"ffmpeg_trim_audio_clip: args: {args}")
    try:
        stdout, stderr = ffmpeg.run(input_stream, capture_stdout=capture_stdout, capture_stderr=silent)
    except Exception as e:
        logging.error(
            f"ffmpeg couldn't trim audio clip. ffmpeg output: \n" + e.stderr.decode("utf-8"))
        raise e
    # return stdout


# buggy, dont use
def ffmpeg_trim_video_clip_directcopy(videofile: Path, timestamp_start: int, timestamp_end: int, quality, outpath: Path,
                                      quiet: bool=True):
    videostream = ffmpeg.input(str(videofile))

    videostream = ffmpeg.output(videostream, str(outpath), ss=timestamp_start/1000, to=timestamp_end/1000, c="copy")

    videostream = ffmpeg.overwrite_output(videostream)
    # args = videostream.get_args()
    # logging.debug(f"ffmpeg_trim_audio_clip: args: {args}")
    try:
        stdout, stderr = ffmpeg.run(videostream, capture_stderr=quiet)
    except ffmpeg._run.Error as e:
        print(e.stderr.decode("utf-8"))
        raise e


# todo: only first audio stream is saved, just want target audio, nothing else
#  https://video.stackexchange.com/a/21738
def ffmpeg_clip_video():
    pass



import setuptools
import pathlib
import os

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
long_description = (HERE / "README.md").read_text()


# from pip's setup.py:
def read(rel_path):
    # type: (str) -> str
    here = os.path.abspath(os.path.dirname(__file__))
    # intentionally *not* adding an encoding option to open, See:
    #   https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    with open(os.path.join(here, rel_path)) as fp:
        return fp.read()


# from pip's setup.py:
def get_version(rel_path):
    # type: (str) -> str
    for line in read(rel_path).splitlines():
        if line.startswith("__version__"):
            # __version__ = "0.9"
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    raise RuntimeError("Unable to find version string.")


setuptools.setup(
    name="subs2cia",
    version=get_version("subs2cia/__init__.py"),
    license='MIT',
    author="Daniel Xing",
    author_email="danielxing97@gmail.com",
    description="A subtitle-based multimedia extractor and compressor",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dxing97/subs2cia",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "ffmpeg-python",
        "pycountry",
        "pysubs2",
        "tqdm",
        "pandas", "tqdm", "gevent", "colorlog"
    ],
    entry_points={
        # create a cli command called 'subs2cia' which runs the main() function in subs2cia.cli
        'console_scripts': ['subs2cia=subs2cia.cli:main', 'subzipper=subs2cia.cli:subzipper_main']
    }
)


from subs2cia.Common import Common, interactive_picker, chapter_timestamps
from subs2cia.sources import AVSFile
from subs2cia.pickers import picker
from subs2cia.sources import Stream
import subs2cia.subtools as subtools
from subs2cia.ffmpeg_tools import export_condensed_audio, export_condensed_video

import logging
from collections import defaultdict
from pathlib import Path
from typing import Union, List


class Condense(Common):
    # todo: shouldn't we just use **kwargs here?
    def __init__(self, sources: [AVSFile], outdir: Path, outstem: Union[str, None], condensed_video: bool,
                 threshold: int, padding: int, partition: int, split: int, demux_overwrite_existing: bool,
                 overwrite_existing_generated: bool, keep_temporaries: bool, target_lang: str, out_audioext: str,
                 minimum_compression_ratio: float, use_all_subs: bool, subtitle_regex_filter: str,
                 subtitle_regex_substrfilter: str, subtitle_regex_substrfilter_nokeep: bool,
                 audio_stream_index: int, subtitle_stream_index: int, ignore_range: Union[List[List[int]], None],
                 ignore_chapters: Union[List[str], None], bitrate: Union[int, None], mono_channel: bool,
                 interactive: bool, no_condensed_subtitles: bool, out_audiocodec: str):
        super(Condense, self).__init__(
            sources=sources,
            outdir=outdir,
            outstem=outstem,
            condensed_video=condensed_video,
            padding=padding,
            demux_overwrite_existing=demux_overwrite_existing,
            overwrite_existing_generated=overwrite_existing_generated,
            keep_temporaries=keep_temporaries,
            target_lang=target_lang,
            out_audioext=out_audioext,
            use_all_subs=use_all_subs,
            subtitle_regex_filter=subtitle_regex_filter,
            # subtitle_regex_substrfilter=subtitle_regex_substrfilter,
            # subtitle_regex_substrfilter_nokeep=subtitle_regex_substrfilter_nokeep,
            audio_stream_index=audio_stream_index,
            subtitle_stream_index=subtitle_stream_index,
            ignore_range=ignore_range,
            ignore_chapters=ignore_chapters,
            bitrate=bitrate,
            mono_channel=mono_channel,
            interactive=interactive,
            out_audiocodec=out_audiocodec
        )
        r"""

        :param sources: List of AVSFile objects, each representing one input file
        :param outdir: Output directory to save to. Default is the directory of the first source file in *sources*
        :param condensed_video: If set, generates condensed video as well
        :param threshold:
        :param padding:
        :param partition:
        :param split:
        :param demux_overwrite_existing: If set, demuxing operations will overwrite existing files on disk
        :param overwrite_existing_generated: If set, generation operations will overwrite existing files on disk
        :param keep_temporaries: If set, will not delete demuxed files on cleanup
        :param target_lang: Target language for audio AND subtitles
        :param out_audioext: Output audio extension
        :param minimum_compression_ratio: Chosen subtitle stream must yield generated audio at least this percent long of audio file
        """

        self.out_subext = None  # extensions must contain dot

        logging.debug(f"Mapping input file(s) {sources} to one output file")

        self.threshold = threshold
        self.partition = partition
        self.split = split

        self.subdata = None
        self.dialogue_times = None
        self.minimum_compression_ratio = minimum_compression_ratio

        self.condensed_audio = True  # can be exposed later if needed
        self.condensed_video = condensed_video
        self.condensed_subtitles = not no_condensed_subtitles  # can be exposed later if needed

        self.subtitle_outfile = None

    def choose_subtitle(self, interactive: bool):
        if len(self.partitioned_streams['subtitle']) == 0:
            logging.warning(f"Couldn't find audio streams in input files")
            return
        k = 'subtitle'
        while True:
            if interactive and len(self.partitioned_streams['subtitle']) > 1:
                self.picked_streams['subtitle'] = interactive_picker(self.sources, self.partitioned_streams, 'subtitle')
            else:

                k = 'subtitle'
                try:
                    self.picked_streams[k] = next(self.pickers[k])
                except StopIteration as s:
                    logging.critical(f"Input files {self.sources} don't contain usable subtitles")
                    self.insufficient = True
                    return
            subfile = self.picked_streams[k].demux(overwrite_existing=self.demux_overwrite_existing)
            if subfile is None:
                logging.warning(f"Error while demuxing {self.picked_streams[k]}")
                self.picked_streams[k] = None
                continue

            assert(self.picked_streams['audio'] is not None)  # must call choose_audio first
            ignore_range = (self.ignore_range or []) + chapter_timestamps(self.picked_streams['audio'].file, self.ignore_chapters or [])
            audiolength = subtools.get_audiofile_duration(self.picked_streams['audio'].demux_file.filepath)
            subdata = subtools.SubtitleManipulator(subfile.filepath,
                                                   threshold=self.threshold, padding=self.padding,
                                                   ignore_range=ignore_range, audio_length=audiolength)
            subdata.load(include_all=self.use_all_subs, regex=self.subtitle_regex_filter,
                         substrreplace_regex='',
                         substrreplace_nokeepchanges=False)
                         # substrreplace_regex=self.subtitle_regex_substrfilter, substrreplace_nokeepchanges=self.subtitle_regex_substrfilter_nokeep)
            if subdata.ssadata is None:
                logging.warning(f"Problem loading subtitle data from {self.picked_streams[k]}")
                self.picked_streams[k] = None
                continue
            subdata.merge_groups()
            times = subdata.get_times()
            ps_times = subtools.partition_and_split(times, self.partition, self.split)

            sublength = subtools.get_partitioned_and_split_times_duration(ps_times)

            compression_ratio = sublength / audiolength
            if compression_ratio < self.minimum_compression_ratio:
                if interactive:
                    resp = input(f"Got compression ratio of {compression_ratio} (ratio of dialogue to total audio), "
                                 f"which is smaller than the minimum"
                                 f" ratio of {self.minimum_compression_ratio}. Continue with picked subtitles? (y/N)")
                    if resp != 'y':
                        continue

                else:
                    logging.warning(f"Got compression ratio of {compression_ratio}, which is smaller than the minimum"
                                 f" ratio of {self.minimum_compression_ratio}"
                                 f"retrying wth different subtitle "
                                 f"file. If too many subtitles are being ignored, try -ni or -R. If the minimum "
                                 f"ratio is too high, try -c.")
                    self.picked_streams[k] = None
                    continue
            self.subdata = subdata
            self.dialogue_times = subtools.partition_and_split(sub_times=times,
                                                               partition_size=1000 * self.partition,
                                                               split_size=1000 * self.split)
            break


    # def process_subtitles(self):
    #     if self.picked_streams['subtitle'] is None:
    #         logging.error(f'No subtitle stream to process for output stem {self.outstem}')
    #         return
    #     if self.insufficient:
    #         return

    def export_subtitles(self):
        if self.picked_streams['subtitle'] is None:
            logging.info(f'No subtitles to process for output {self.outstem}')
            return
        subpath = self.picked_streams['subtitle'].get_data_path()
        subext = subpath.suffix
        self.subdata.condense()

        self.subtitle_outfile = Path(self.outdir) / (
                    self.outstem + f'.condensed{self.out_subext if self.out_subext is not None else subext}')
        self.subdata.condensed_ssadata.save(str(self.subtitle_outfile), encoding=u'utf-8')
        logging.info(f"Wrote condensed subtitles to {self.subtitle_outfile}")

    def export_audio(self):
        if self.picked_streams['audio'] is None:
            logging.error(f'No audio stream to process for output stem {self.outstem}')
            return
        outfile = Path(self.outdir) / (self.outstem + f'.{self.out_audioext}')
        # logging.info(f"exporting condensed audio to {outfile}")  # todo: fix output naming
        if outfile.exists() and not self.overwrite_existing_generated:
            logging.warning(f"Can't write to {outfile}: file exists and not set to overwrite")
            return
        export_condensed_audio(self.dialogue_times, audiofile=self.picked_streams['audio'].get_data_path(),
                               outfile=outfile, to_mono=self.to_mono, quality=self.quality, codec=self.out_audiocodec)
        # logging.info(f"Wrote condensed audio to {outfile}")

    def export_video(self):
        if self.picked_streams['video'] is None:
            logging.error(f'No video stream to process for output stem {self.outstem}')
            return

        outfile = Path(self.outdir) / (self.outstem + '.mkv')
        # logging.info(f"exporting condensed video to {outfile}")
        if outfile.exists() and not self.overwrite_existing_generated:
            logging.warning(f"Can't write to {outfile}: file exists and not set to overwrite")
            return
        export_condensed_video(self.dialogue_times, audiofile=self.picked_streams['audio'].get_data_path(),
                               subfile=self.subtitle_outfile,
                               videofile=self.picked_streams['video'].get_data_path(),
                               outfile=outfile)
        logging.info(f"Wrote condensed video to {outfile}")
        return

    def export(self):
        if self.insufficient:
            return
        subtools.get_compression_ratio(self.dialogue_times, self.picked_streams['audio'].demux_file.filepath)
        if self.condensed_subtitles:
            self.export_subtitles()
        if self.condensed_audio:
            self.export_audio()
        if self.condensed_video:
            self.export_video()
